function Format-Number($n) {
    $prefix = @("", "k", "M", "G", "T", "P", "E", "Z", "Y")
    $index = 0
    while($n -ge 1024 -and $index -lt $prefix.Length - 1) {
        $n /= 1024
        $index += 1
    }
    return "{0:N2} {1}" -f $n, $prefix[$index]
}

function ctEruit {
        Write-Host "`r`n`r`n(Druk op Enter om te beindigen)" -NoNewline
        Read-Host
        exit
}

$identity = [System.Security.Principal.WindowsIdentity]::GetCurrent()
$princ = New-Object System.Security.Principal.WindowsPrincipal($identity)
if(-not $princ.IsInRole([System.Security.Principal.WindowsBuiltInRole]::Administrator)) {
    Write-Host "Fout: dit script moet met administratorrechten draaien!"
    ctEruit
}

Clear-Host
Write-Host "`r`n            Welkom bij c't-WIMage!"
Write-Host "`r`nOp het station waarvan c't-WIMage gestart wordt, wordt een nieuwe partitie aangemaakt waar alle bestanden op worden gekopieerd die voor c't-WIMage nodig zijn."


       

$myDriveLetter = ($MyInvocation.MyCommand.Definition).Substring(0, 1)
$myDriveName = (Get-Volume -DriveLetter $mydriveletter).FileSystemLabel
$myPart = Get-Partition -DriveLetter $myDriveLetter
$myDisk = Get-Disk -Partition $myPart
if ((get-volume -driveletter ${myDriveLetter}).drivetype -eq "Removable") {
    if ((Get-ItemProperty "HKLM:\SOFTWARE\Microsoft\Windows NT\CurrentVersion\" -Name ReleaseID -ErrorAction SilentlyContinue).ReleaseId -lt 1703) {
        Write-Host "`r`nFout: deze Windows-versie biedt geen ondersteuning voor meerdere partities op een USB-stick. Gebruik een USB-harde schijf of USB-ssd."
        ctEruit
        }
    }
$free = $myDisk.LargestFreeExtent
if($free -lt 3MB) {
    Write-Host "`r`nFout: op het station is geen ongepartitioneerde vrije ruimte."
    ctEruit
}

if (-not (Test-Path -Path ${myDriveLetter}:\x64\sources)) {
     if (-not (Test-Path -Path ${myDriveLetter}:\sources)) {
         Write-Host "`r`nFout: er zijn geen voor c't-WIMage geschikte set-upbestanden op het station aangetroffen."
         Write-Host "        Lees eerst de handleiding bij dit script in c't 10/2023!"
         ctEruit
    }
}

$message  = "`r`nOp de gegevensdrager $($myDisk.DiskNumber) (`"$($myDisk.FriendlyName)`" op $($myDisk.BusType); waar ook "
$message += "het station ${myDriveletter}: (`"${mydrivename}`") op staat, wordt een nieuw volume van $(Format-Number $free)bytes aangemaakt."
$message += "`r`n`r`nWilt u doorgaan?"
$choiceYes = New-Object System.Management.Automation.Host.ChoiceDescription "&Ja", "Ja: de partitie wordt zoals aangegeven aangemaakt"
$choiceNo = New-Object System.Management.Automation.Host.ChoiceDescription "&Nee", "Nee: de bewerking nu stoppen"
$choices = [System.Management.Automation.Host.ChoiceDescription[]]($choiceYes, $choiceNo)
$confirm = $Host.UI.PromptForChoice("We gaan aan de slag", $message, $choices, 0)
if($confirm -ne 0) {
    Write-Host "`r`nDoor gebruiker gestopt"
    ctEruit
}

$newPart = $myDisk | New-Partition -UseMaximumSize -MbrType IFS
if($newPart) {
    $newVolume = $newPart | Format-Volume -FileSystem NTFS -NewFileSystemLabel 'CT-WIMAGE'
    if($newVolume) {
        # Wijs de volgende vrije stationsletter na de huidige toe
        $myDriveLetter = [char]($myDriveLetter.ToUpper())
        $newLetter = $myDriveLetter
        do {
            $newLetter = [char]([int]($newLetter) + 1)
            if($newLetter -gt [char]'Z') {
                # Na 'Z' ook de letters vanaf 'C' proberen
                $newLetter = [char]'C'
            }
            if($newLetter -eq $myDriveLetter) {
                # Alle letters geprobeerd, geen vrije letters aangetroffen
                [system.windows.forms.messagebox]::show("`r`nDe schijf is aangemaakt maar er is geen vrije stationsletter gevonden.")
                exit
            }
            $testVolume = Get-Volume -DriveLetter $newLetter -ErrorAction SilentlyContinue
        } while ($testVolume)
        Start-Sleep -Seconds 1
        $newPart | Set-Partition -NewDriveLetter $newLetter
        Get-Volume -DriveLetter $myDriveLetter | Set-Volume -NewFileSystemLabel "ct-Boot"
        Write-Host "`r`nStation ${newLetter}: ($(Format-Number $newVolume.Size)Byte, `"$($newVolume.FileSystemLabel)`") is succesvol aangemaakt."
        Rename-Item -Path ${myDriveLetter}:\autorun.inf -NewName autorun.org -ErrorAction SilentlyContinue
    }
    else {
        Write-Host "`r`nFout: de nieuwe partitie kon niet worden geformatteerd."
        ctEruit
    }
}
else {
    Write-Host "`r`nFout: de partitie kon niet worden gemaakt."
    ctEruit
}

Write-Host "`r`nBestanden worden gekopieerd..."

function Copy-Wimage($Arch, $ctNLPad) {
    if(-not $ctNLPad) { $ctNLPad = $Arch + "\" }
    Copy-Item -Path "${myDriveLetter}:\${ctNLPad}sources" -Destination "${newLetter}:\${ctNLPad}sources" -Recurse -Exclude Boot.wim,install.esd
        # Zekere voor het onzekere: oudere PowerShell-versies negeren soms de -Exclude-parameter
        ("Boot.wim","install.esd") | % { Remove-Item "${newLetter}:\${ctNLPad}sources\${_}" -ErrorAction SilentlyContinue }
    Rename-Item -path "${myDriveLetter}:\${ctNLPad}sources" -NewName sources.org
    New-Item -itemtype "directory" -path "${myDriveLetter}:\${ctNLPad}sources" | Out-Null
    Copy-Item -path "${myDriveLetter}:\${ctNLPad}sources.org\boot.wim" -destination "${myDriveLetter}:\${ctNLPad}sources\"
    Copy-Item -Path "${myDriveLetter}:\ct-WIMage\ct-WIMage.${Arch}" -Destination "${newLetter}:\${ctNLPad}ct-WIMage-${Arch}.bat"
    Copy-Item -Path "${myDriveLetter}:\ct-WIMage\vshadow.${Arch}" -Destination "${newLetter}:\${ctNLPad}sources\vshadow${Arch}.exe"
    Copy-Item -Path "${myDriveLetter}:\ct-WIMage\ct-WIMage.ini" -Destination "${newLetter}:\${ctNLPad}sources\ct-WIMage.ini"
    Copy-Item -Path "${myDriveLetter}:\ct-WIMage\ei.cfg" -Destination "${newLetter}:\${ctNLPad}sources\ei.cfg"
}

   
if (Test-Path -Path ${myDriveLetter}:\sources) {
    if( (Get-Content "${myDriveLetter}:\sources\idwbinfo.txt" -Raw) -match "BuildArch=(\w+)") {
        $archtemp = $Matches[1]
            if ($archtemp -eq "amd64") {
                $arch = "x64"
                }
                elseif ($archtemp -eq "x86") {
                $arch = "x86"
                }
            else {
                Write-Host "Fout: platform-identificatie $archtemp van de installatiebestanden op station ${myDriveLetter}: wordt niet ondersteund."
                ctEruit
                }
            }
    else {
        Write-Host "Fout: geen platform-identificatie op station ${myDriveLetter}: aangetroffen. Raadpleeg de handleiding in c't 10/2023!"
        ctEruit
    }
Copy-Wimage $arch "\"
}

if (Test-Path -Path ${myDriveLetter}:\x64\sources) {
    Copy-Wimage "x64"
    Copy-Wimage "x86"
    Copy-Item -Path "${myDriveLetter}:\ct-WIMage\ct-WIMage.start" -destination "${newLetter}:\ct-WIMage-x86+x64.bat"
    Copy-Item -Path "${myDriveLetter}:\ct-wimage\bcd.bios" -destination "${myDriveLetter}:\boot\bcd"
    Copy-Item -Path "${myDriveLetter}:\ct-wimage\bcd.uefi" -destination "${myDriveLetter}:\efi\microsoft\boot\bcd"
}

Write-Host "`r`n`r`nKlaar!"
ctEruit